import { ConfigModel, Config } from '../models/model.ts';
import { CustomError } from '@kevisual/router';
import { redis } from '@/app.ts';
import { User } from '@/models/user.ts';
import { UserPermission, UserPermissionOptions } from '@kevisual/permission';

export class ShareConfigService extends ConfigModel {
  /**
   * 获取分享的配置
   * @param key 配置的key
   * @param username 分享者的username
   * @returns 配置
   */
  static async getShareConfig(key: string, username: string, options: UserPermissionOptions) {
    const shareCacheConfigString = await redis.get(`config:share:${username}:${key}`);
    let shareCacheConfig: Config;
    try {
      shareCacheConfig = JSON.parse(shareCacheConfigString);
    } catch (e) {
      await redis.set(`config:share:${username}:${key}`, '', 'EX', 0); // 删除缓存
      throw new CustomError(400, 'config parse error');
    }
    const owner = username;
    if (shareCacheConfig) {
      const permission = new UserPermission({ permission: shareCacheConfig?.data?.permission, owner });
      const result = permission.checkPermissionSuccess(options);
      if (!result.success) {
        throw new CustomError(403, 'no permission');
      }
      return shareCacheConfig;
    }
    const user = await User.findOne({
      where: { username },
    });
    if (!user) {
      throw new CustomError(404, 'user not found');
    }
    const config = await ConfigModel.findOne({
      where: { key, uid: user.id },
    });
    if (!config) {
      throw new CustomError(404, 'config not found');
    }
    const permission = new UserPermission({ permission: config?.data?.permission, owner });
    const result = permission.checkPermissionSuccess(options);
    if (!result.success) {
      throw new CustomError(403, 'no permission');
    }
    await redis.set(`config:share:${username}:${key}`, JSON.stringify(config), 'EX', 60 * 60 * 24 * 7); // 7天
    return config;
  }
  static async expireShareConfig(key: string, username: string) {
    if (key && username) {
      await redis.set(`config:share:${username}:${key}`, '', 'EX', 1);
    }
  }
}
